"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
const query_string_1 = require("query-string");
const matchPath_1 = require("./matchPath");
const generatePath_1 = require("./generatePath");
exports.UNKNOWN_ROUTE = '@@direct-react-router/UNKNOWN_ROUTE';
function parseLocation({ routes }, { pathname, search, hash }) {
    const query = query_string_1.parse(search);
    let [location, ...others] = Object.keys(routes).reduce((prev, key) => {
        const matched = matchPath_1.matchPath(pathname, {
            path: routes[key],
            exact: true
        });
        matched &&
            prev.push({
                pathname,
                search,
                hash,
                key,
                params: matched.params,
                query
            });
        return prev;
    }, []);
    if (others.length) {
        throw new Error(`Ambiguous route match: ${[location, ...others].map(x => x.key).join(', ')}`);
    }
    return (location || {
        pathname,
        search,
        hash,
        key: exports.UNKNOWN_ROUTE,
        params: {},
        query
    });
}
exports.parseLocation = parseLocation;
function generateUrl({ routes }, { routeKey, params, query, hash }) {
    const route = routes[routeKey];
    if (!route) {
        throw new Error('Unknown route key');
    }
    let url = generatePath_1.generatePath(route, params);
    if (query) {
        const qs = query_string_1.stringify(query);
        qs && (url += '?' + qs);
    }
    if (hash) {
        if (hash[0] !== '#') {
            throw new Error('Hash must be started with #');
        }
        url += hash;
    }
    return url;
}
exports.generateUrl = generateUrl;
